"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Class list.
 */
class HTMLCollection extends Array {
    constructor() {
        super(...arguments);
        this._namedItems = {};
    }
    /**
     * Returns item by index.
     *
     * @param index Index.
     */
    item(index) {
        return index >= 0 && this[index] ? this[index] : null;
    }
    /**
     * Returns named item.
     *
     * @param name Name.
     * @returns Node.
     */
    namedItem(name) {
        return this[name] || null;
    }
    /**
     * Appends named item.
     *
     * @param node Node.
     * @param name Name.
     */
    _appendNamedItem(node, name) {
        if (name) {
            this._namedItems[name] = this._namedItems[name] || [];
            if (!this._namedItems[name].includes(node)) {
                this._namedItems[name].push(node);
            }
            if (!this[name]) {
                this[name] = this._namedItems[name][0];
            }
        }
    }
    /**
     * Appends named item.
     *
     * @param node Node.
     * @param name Name.
     */
    _removeNamedItem(node, name) {
        if (name && this._namedItems[name]) {
            const index = this._namedItems[name].indexOf(node);
            if (index > -1) {
                this._namedItems[name].splice(index, 1);
                if (this._namedItems[name].length === 0) {
                    delete this._namedItems[name];
                    delete this[name];
                }
                else {
                    this[name] = this._namedItems[name][0];
                }
            }
        }
    }
}
exports.default = HTMLCollection;
//# sourceMappingURL=HTMLCollection.js.map